@@ -1,19 +1,20 @@
 /* Guile-Avahi --- Guile bindings for Avahi.
-   Copyright (C) 2007  Ludovic Courts <ludovic.courtes@laas.fr>
+   Copyright (C) 2007  Ludovic Courts <ludo@gnu.org>
 
-   Guile-Avahi is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
+   This file is part of Guile-Avahi.
 
-   Guile-Avahi is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
+   Guile-Avahi is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at
+   your option) any later version.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with Guile-Avahi; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA  */
+   Guile-Avahi is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+   General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -206,7 +207,9 @@ SCM_SMOB_MARK (scm_tc16_avahi_watch, mark_avahi_watch, watch)
 
   c_watch = scm_to_avahi_watch (watch, 1, __FUNCTION__);
 
-  return (c_watch->stuff);
+  scm_gc_mark (c_watch->stuff);
+
+  return (c_watch->guile_poll->poll_smob);
 }
 
 /* Mark the user data associated with TIMEOUT.  */
@@ -216,7 +219,9 @@ SCM_SMOB_MARK (scm_tc16_avahi_timeout, mark_avahi_timeout, timeout)
 
   c_timeout = scm_to_avahi_timeout (timeout, 1, __FUNCTION__);
 
-  return (c_timeout->stuff);
+  scm_gc_mark (c_timeout->stuff);
+
+  return (c_timeout->guile_poll->poll_smob);
 }
 
 
@@ -421,6 +426,8 @@ avahi_guile_poll_new (SCM new_watch, SCM update_watch_x, SCM free_watch,
   guile_poll->free_timeout = free_timeout;
   guile_poll->update_timeout_x = update_timeout_x;
 
+  guile_poll->poll_smob = SCM_BOOL_F;
+
   return (guile_poll);
 }
 
